function s = fair_spread(tranche, properties, index_quote, coupon_times, n, delta, r, quoted_upfront, running_spread)
% funkce pocita ferovy spread pro transe, pro ktere zname jen upfront a running spread, tj. dela konverzi: upfront + fixed running spread -> fair spread
% neni k tomu potreba znat impikovanou korelaci
% vypocet funguje tak, ze se nastavi model na Gaussovskou copulu (properties.name = 'Normal'), pak se dopocita implikovana korelace, ktera dava stejny ufront jako kotovany upfront
% a jakmile mame tuto implikovanou korelaci, tak s ni spocitame fair spread

%% vstupy
% tranche       = vektor dvou hodnot [attachment detachment] transe, pro kterou hledame fair spread
% properties    = objekt (calass) s nastavenim rozdeleni a integrace. Musi obsahovat nasledujici promenne
    % .xMin     = dolni integral integrace systematickeho faktoru X
    % .xMax     = horni integral integrace systematickeho faktoru X
    % .nPoints  = pocet bodu integrace
% index quote   = kotace indexu dane iTraxx/CDX serie 
% coupon_times  = vektor casu kuponu CDO. Kazde CDO (i transe deal) ma fixni maturitu
% n             = pocet CDO jmen v kosi (vetsinou 125)
% delta         = recovery kazde entity v kosi
% r             = prumerna risk-free sazba pro diskont cash-flow v dealu
% quoted_upfront= kotovany upfront pro danou transi (ze screenu z Bloombergu)
% running_spread= fixed running spread, ktery je dan ke kotovanemu upfrontu

%% vypocet
% ve properties nastavime model na Normal, tj. Gaussovskou copulu. Pro tuto pak budeme hledat implikovanou korelaci dane transe, ktera produkuje stejny upfront jako kotovany
% jamkmile mame tuto korelaci, tak spocitame fair spread

properties.name = 'Normal'; 
f = @(rho) (find_diff(tranche, properties, index_quote, coupon_times, rho, n, delta, r, quoted_upfront, running_spread));
rho = fzero(f, 0.5);
s = CDO_price(tranche, properties, index_quote, coupon_times, rho, n, delta, r);

%% funkce primo vyuziva funkci 
% CDO_price(tranche, properties, index_quote, coupon_times, rho, n, delta, r)

%% funkce neprimo vyuziva funkci 
% lgwt(N,a,b)
% get_expected_tranche_principal(tranche, properties, x, lambda, t, rho, n, delta)
% get_copula_prob(properties, x, lambda, t, rho)
end

function difference = find_diff(tranche, properties, index_quote, coupon_times, rho, n, delta, r, quoted_upfront, running_spread)
% optimalizacni funkce pro dohledani optimalni korelace, ktera zajisti modelovy upfront = kotovany upfront

[s, ~, ~, C] = CDO_price(tranche, properties, index_quote, coupon_times, rho, n, delta, r);

model_upfront = C*(1-running_spread/s);
difference = model_upfront - quoted_upfront;

%quoted_spead = running_spread*C/(C-quoted_upfront);
%difference = s - quoted_spead;
end