function Q = get_copula_prob(properties, x, lambda, t, rho)
% funkce pocita pst defaultu Q(t|X=x) = Pr[tau<=t|X=x]) na zaklade faktorove funkce a zafixovani spolecneho faktoru X=x
% funkce umi pocitat tuto pst pro faktorove specifikace: 'Normal' (Gaussian copula), 'NIG', 'double_t', 'Normal_stochastic'

%% vstupy
% properties        = objekt (calass) s nastavenim rozdeleni a integrace. Musi obsahovat nasledujici promenne
    % .name         = charakterizace faktorove fce: 'Normal' (Gaussian copula), 'NIG', 'double_t', 'Normal_stochastic'
    % .NIG_alpha    = parametr alpha u NIG rozdeleni (NIG specifikace)
    % .double_t_nu  = parametr stupnu volnosti u t rozdeleni (double_t) specifikace  
    % .normal_stochastic_p = parametr p u normal_stochastic
    % .normal_stochastic_q = parametr q u normal_stochastic
% x                 = skalar nebo vektor hodnot fixovaneho common faktoru X=x
% lambda            = hazard rate ziskani z kotace iTraxx/CDX indexu
% t                 = casovy horizont, pro ktery pocitame pst. Q(t|X=x) = Pr[tau<=t|X=x]
% rho               = parametr korelace, poziva se u specifikaci vsech modelu

%% vypocet
PD_uncond = 1- exp(-lambda*t);
if strcmp(properties.name, 'Normal')
    Q = normcdf((norminv(PD_uncond) - sqrt(rho)*x)/sqrt(1-rho));
elseif strcmp(properties.name, 'NIG')
    Z_thr = NIG_quantile(PD_uncond, properties.NIG_alpha/sqrt(rho), 0, 0, properties.NIG_alpha/sqrt(rho));
    Q = NIG_CDF( (Z_thr - sqrt(rho)*x)./sqrt(1-rho), properties.NIG_alpha*sqrt(1/rho - 1), 0, 0, properties.NIG_alpha*sqrt(1/rho - 1));
elseif strcmp(properties.name, 'NIG_ext')
    alpha = properties.NIG_alpha; beta = properties.NIG_beta; gam = sqrt(alpha^2 - beta^2);
    Z_thr = NIG_quantile(PD_uncond, alpha/sqrt(rho), beta/sqrt(rho), -1/sqrt(rho)*beta*gam^2/alpha^2, sqrt(1/rho)*gam^3/alpha^2);
    Q = NIG_CDF( (Z_thr - sqrt(rho)*x)./sqrt(1-rho), alpha*sqrt(1/rho-1), beta*sqrt(1/rho-1), -sqrt(1/rho -1)*beta*gam^2/alpha^2, sqrt(1/rho-1)*gam^3/alpha^2);
elseif strcmp(properties.name, 'double_t')
    nu = properties.double_t_nu;
    Z_thr = double_t_quantile(PD_uncond, nu, rho);
    Q = tcdf( (Z_thr - sqrt(rho * (nu-2)/nu)*x)./sqrt( (1-rho) * (nu-2)/nu),  nu);
elseif strcmp(properties.name, 'double_t_ext')
    nuX = properties.double_t_nu; nuY = properties.double_t_nuY; %mX = sqrt((nuX-2)/nuX); mY = sqrt((nuY-2)/nuY);
    Z_thr = double_t_quantile_ext(PD_uncond, nuX, nuY, rho);
    Q = tcdf( (Z_thr - sqrt(rho * (nuX-2)/nuX)*x)./sqrt( (1-rho) * (nuY-2)/nuY),  nuY);
elseif strcmp(properties.name, 'Normal_stochastic')
    p = properties.normal_stochastic_p;
    q = properties.normal_stochastic_q;
    Q = (1-p)*((1-q)*normcdf((norminv(PD_uncond) - sqrt(rho)*x)/sqrt(1-rho)) + q*normcdf(norminv(PD_uncond))) + p*(x <= norminv(PD_uncond));
elseif strcmp(properties.name, 'Clayton')
    theta = properties.clayton_theta;
    Q = exp(x*(1-1/PD_uncond^theta));
elseif strcmp(properties.name, 'Gumbel')
    theta = properties.gumbel_theta;
    Q = exp(-x* (-log(PD_uncond))^theta);
elseif strcmp(properties.name, 'Gamma_exponential')
    alpha = properties.gamma_exponential_alpha;
    beta = properties.gamma_exponential_beta;
    q = 1-1/(1+beta*t)^(alpha);
    Q = normcdf((norminv(q) - sqrt(rho)*x)/sqrt(1-rho));
end
%% funkce primo vyuziva funkci 
% NIG_quantile(p, alpha, beta, mu, delta)
% NIG_CDF(x, alpha, beta, mu, delta)
% double_t_quantile(p, nu, rho)

%% funkce neprimo vyuziva funkci
% --
end