function E = get_expected_tranche_principal(tranche, properties, x, lambda, t, rho, n, delta)
% funkce ziskava expected tranche pricipal v nejakem case t, kde t muze byt vektor (v tomto vektoru nesmi byt aktualni cas t=0. E pro cas 0 je 1 a prida se vzdy automaticky)
% nejbeznejsi pripad je, ze x je vektor a t je taky vektor, v takovem pripade bude E matice expected tranche principal pro vsechny casy a vsechny hodnoty systematickeho faktoru X

%% vstupy
% tranche           = vektor dvou hodnot [attachment detachment] transe, pro kterou hledame fair spread
% properties        = objekt (calass) s nastavenim rozdeleni a integrace. Musi obsahovat nasledujici promenne
    % .name         = charakterizace faktorove fce: 'Normal' (Gaussian copula), 'NIG', 'double_t', 'Normal_stochastic'
    % .NIG_alpha    = parametr alpha u NIG rozdeleni (NIG specifikace)
    % .double_t_nu  = parametr stupnu volnosti u t rozdeleni (double_t specifikace)  
    % .normal_stochastic_p = parametr p u normal_stochastic
    % .normal_stochastic_q = parametr q u normal_stochastic
% x                 = skalar nebo vektor hodnot fixovaneho common faktoru X=x
% lambda            = hazard rate ziskani z kotace iTraxx/CDX indexu
% t                 = jeden cas nebo vektor casu, pro ktery chceme spocitat expected tranche principal
% rho               = parametr korelace, poziva se u specifikaci vsech modelu 
% n                 = pocet CDO jmen v kosi (vetsinou 125)
% delta             = recovery kazde entity v kosi

%% vypocet
tranche(2) = min(1.0-delta, tranche(2)); % uprava pro super-seniorni transi. 
                                       % Celkova ztrata portfolia nikdy nemuze byt napr. 100 %, protoze recovery > 0 (pri vsech defultech vzdy neco zustane). Max ztrata portfolia je LGD
n_attach = ceil((n*tranche(1))/(1-delta)); % pocet defaultu, kde zacne transe byt afektovana
n_detach = ceil((n*tranche(2))/(1-delta)); % pocet defaultu, kde transe bude jiz kompletne vymazana

% if, ktery resi, zda je t vektor nebo skalar. Pokud je t vektor, pro kazdy cas ve vektoru t se bude se bude tato funkce rekurzive volat
if (numel(t) == 1)
    % zjistime podminene-nezavislou pst default libovolneho jmena v kosi
    Q = get_copula_prob(properties, x, lambda, t, rho);

    Q_mat = zeros(numel(x),n+1);
    Q_mat(:,[1 2]) = [1-Q(:) Q(:)];
    Q_mat_prob = ifft(fft(Q_mat, [], 2).^n, [], 2);
    E = sum(Q_mat_prob(:,1:n_attach), 2) + sum( Q_mat_prob(:,n_attach+1:n_detach).*  repmat(1- ((n_attach:n_detach-1)*(1-delta)/n - tranche(1))/(tranche(2) - tranche(1)), numel(x),1 ), 2);
else % rekurzivni volani fce pro jeden vybrany cas t(i)
   for i = 1:numel(t); E(:,i) = get_expected_tranche_principal(tranche, properties, x, lambda, t(i), rho, n, delta); end
   E = [ones(numel(x), 1) E]; % jakmile je oditerovano, prida se stav E(:,1) = 1, protoze vzdy na zacatku t = 0 neni nic v defaultu a transe je tedy cela
end

%% funkce primo vyuziva funkci 
% get_copula_prob(properties, x, lambda, t, rho)

%% funkce neprimo vyuziva funkci 
% NIG_quantile(p, alpha, beta, mu, delta)
% NIG_CDF(x, alpha, beta, mu, delta)
% double_t_quantile(p, nu, rho)
end