function [An, P] = loss_distrib(exposures, PD, LGD, unit)
% funkce je implementaci modelu CreditRisk+ konstantnimi default raty, dluznici jsou tedy nezavisli
% vystupem je An = vektor pst ztrat a P vektor psti poctu defaultu

%% vstupy
% exposures = vektor expozic (nijak neupravenych o LGD) a v peneznich jednotkach, ne v unitech
% PD = psti defaultu
% LGD = LGD, ktere se aplikuje na expozice, aby se ziskala jen cast, kterou ztratime pri defaultu
% unit = pocet jednotek, ve kterych se bude vyjadrovat ztrata

% ***** zakladni transformace vstupnich dat na modelova data **********
% pripravi se (i) nu = jakasi zaokrouhlena defaultovatelna expozice v jednotkach unit a (ii) epsilon, ocekavana ztrata v jednotkach unit
nu = ceil(exposures.*LGD/unit);    % pocet jednotek ztraty v pripade defaultu (tj. 'defaultovatelna expozice') (zaokrouhleno na cela cisla)
                                   % toto cislo (nu) je v technickem dokumentu oznaceno jen jako pocet jednotek 'expozice' (eq. (11))
                                   % maximalni dosazitelna ztrata v portfoliu (v jednotkach) je tedy sum(nu)
epsilon = (PD.*LGD.*exposures)/unit; % ocekavana ztrata vyjadrena v jednotkach (nezaokrouhluje se)
% **********************************************************************

% ********************** priprava dat pro 'expozicni' pasma dle vzorce (12) ********************
% vytvoreni expzocnich pasem j = 1, ..., m
% nu_band = distinct nu (tj. vsechny mozne expozice)
% epsilon_band = suma epsilonu v jednotlivych expozicnich pasmech
[nu_band, epsilon_band, mu_band] = exposure_bands(nu, epsilon);

% pst zadne ztraty = pst 0 defaultu (dle vzorce (10) nebo (26))
An = zeros(sum(nu)+1, 1);
An(0 +1) = exp(-sum(mu_band));

% rekurze pro ztraty n >= 1, vzorec (24)
for n = 1:sum(nu)
   j_indices = (nu_band <= n+0.00001);
   nu_band_selection = nu_band(j_indices);
   
   epsilon_selection = epsilon_band(j_indices);
   A_selection = An(n-nu_band_selection +1);
   
   An(n +1) = 1/n* sum(epsilon_selection.*A_selection);
end

% vypocet vektor psti defaultu P(i +1) = Pr[pocet defaultu = i] dle vzorce (10)
% pocet defaultu v portfoliu ma exponencialni rozdeleni
for j = 0:numel(nu); P(j +1) = exp(-sum(mu_band))*sum(mu_band)^j/factorial(j); end;

%% funkce primo vyuziva funkci 
% exposure_bands(nu, epsilon)

%% funkce neprimo vyuziva funkci
% --
end