function [G, relative_losses, max_loss, EL, UL, ETL, loss999q] = loss_distr_rnd_RR_double_t(E, PD, rho_PD, rho_RR, alpha_RR, beta_RR, unit, nu)
% funkce vraci loss-PGF pres cele portfolio, pokud uvazujeme double t copula model se stochastickou korelaci PD i RR
% dale vraci:
%   relative_losses = vektor ztrat vztazeny k celemu nominalu expozice
%   max_loss        = maximalni mozna dosazitelna ztrata v jednotkach unit
%   EL              = ocekavana ztrata (relativne), 99.9 % kvantil
%   UL              = neocekvana ztrata (relativne), 99.9 % kvantil
%   ETL             = tail loss (relativne), 99.9 % kvantil

% POZOR funkce neumi resit pripad, kdy by 2 a vice recovery cohorty generovaly stejnou ztratu. To zpravidla nastane, pokud je unit velke cislo nebo thresholdy moc u sebe
% napriklad: mame expozici 3 a recovery delta_j = [0.05, 0.1, ...]. Pak ztraty odpovidajici temto recovery bude ceil((1-0.05)*3) = 3, ceil((1-0.1)*3) = 3, ...
% s timto si funkce neumi poradit a predpoklada, ze ztraty v ruznych recovery kohortach jsou ruzna cisla 

%% vypocet
[x, w] = lgwt(500,-40,40); % x a w pro Gauss-Legendre quadraturu

% matice podminenych psti defaultu Qi(x). Radky odpovidaji ruznym X=x_k, sloupce dluznikum i = 1, ..., n
Q_mat = Q_fixed_PD_double_t(PD, rho_PD, x, nu);

% loss-PGF podminena X=x_k. Radky jsou ruzna x_k, sloupce ruzne ztraty v peneznich jednotkach  
tic;
Gx = get_loss_vec_rnd_RR_double_t(E, x, Q_mat, rho_RR, alpha_RR, beta_RR, unit, nu, 50);
toc;
% integrace loss-PGF|X=x pres hustotu X
G = sum(repmat(tpdf(x(:), nu), 1, size(Gx, 2)).*repmat(w(:), 1,size(Gx, 2)).*Gx); G(G<0) = 0;

% ziskani vystupnich statistik
cumProbs = cumsum(G);
losses = 0:numel(G)-1;
relative_losses = losses/sum(E/unit);
max_loss = losses(end);

idx999q = find(cumProbs>0.999, 1, 'first');
EL = sum(relative_losses.*G);
loss999q = relative_losses(idx999q);
UL = loss999q - EL;
ETL = sum(relative_losses(idx999q+1:end).*G(idx999q+1:end))/0.001;

%% funkce primo vyuziva fci
% get_loss_vec_rnd_RR_double_t(E, x, Q_mat, rho_RR, alpha_RR, beta_RR, unit, nu, cluster_size)
% Q_fixed_PD_double_t(PD, rho_PD, x, nu)

%% funkce neprimo vyuziva fci
% prepare_RR_thresholds(alpha_RR, beta_RR)
% get_prob_RR_double_t(x, rho_RR, RR_thresholds, FZi_inv, nu)
% double_t_quantile(p, nu, rho)
end