function sigma = BDT_calib_to_caps_floors(ycrv_rates, ycrv_maturities, quote_ivol, quote_R, cap_floor_maturities, tenor_LIBOR, freq_discount, caps_floors)
% funkce kalibruje BDT model na trzni kotace capu / flooru
% vystupem je parametr sigma BDT modelu

% optimalizace pracuje s hledanim nejlepsiho fittu na ceny capu / flooru, ktere ziskame Blackovym modelem pro capy / floory (resp caplety a floorlety) 
% nejlepsi fit je ve smyslu nejmensi celkove relativni chyby, tedy suma( abs(cena_BDT_model(i)/cena_Black(i) - 1) ), pres vsechna i. Kazde i je jeden cap / floor z vektoru (quote_ivol a quote_R)

%% vstupy
% ycrv_rates            = vektor urokovych sazeb pozorovanych na trhu
% ycrv_maturities       = splatnosti pro ycrv_rates
% quote_ivol            = kotace blackovych implikovanych volatilit pro capy / floory (vektor). 
% quote_R               = cap / floor rate capu / flooru (vektor). Pokud jsou ve vektoru nekde hodnoty <= 0, rikame tim, ze chceme kalibrovat na ATM kotace (velmi casto ATM strike neni poskytovan)
%                         tato funkce si umi ATM cap / floor rate R dopocitavat
% cap_floor_maturities  = delka cap / floor kontraktu (v letech). Napr. 5 let 
% tenor_LIBOR           = tenor LIBOR sazby (v letech), na ktery se vztahuje cap/floor. Nejcasteji 3M = 0.5, 6M = 0.5
% freq_discount         = frekvence uroceni pro spocitani diskontnich faktoru z vynosove krivky ycrv_rates
% caps_floors           = nastavime 1, pokud chcem kalibrovat na capy. Cokoliv jineho, pokud chceme kalibrovat na floory

%% vypocet
f = @(params)(optim_fun(params, ycrv_rates, ycrv_maturities, quote_ivol, quote_R, cap_floor_maturities, tenor_LIBOR, freq_discount, caps_floors));
params = fminunc(f, 0.45);

[err, cap_floor_val, cap_floor_val_quote] = optim_fun(params, ycrv_rates, ycrv_maturities, quote_ivol, quote_R, cap_floor_maturities, tenor_LIBOR, freq_discount, caps_floors);
sigma = params;

% vykreslime grafy
scatter(cap_floor_val_quote*10000, cap_floor_val*10000); xlabel('cap premia - market (bps)'); ylabel('cap premia - BDT model (bps)'); grid on; hold on;
plot((0:0.01:0.3)*10000, (0:0.01:0.3)*10000, 'r--'); hold off;

%% funkce primo vyuziva funkci
% optim_fun(params, ycrv_rates, ycrv_maturities, quote_ivol, quote_R, cap_floor_maturities, tenor_LIBOR, freq_discount)... zde v m. filu
% black_cap_floor(R, T, tenor_LIBOR, sigma, cap_floor, ycrv_rates, ycrv_maturities, freq_discount)
% BDT_cap_floor(R, T, tenor_LIBOR, sigma, cap_floor, ycrv_rates, ycrv_maturities, freq_discount)

%% funkce neprimo vyuziva funkci
% black_caplet_floorlet(R, T, tenor_LIBOR, sigma, caplet_floorlet, ycrv_rates, ycrv_maturities, freq_discount)
% get_LIBOR_rate(T, tenor_LIBOR, ycrv_rates, ycrv_maturities, freq_discount)
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
% BDT(ycrv_rates, ycrv_maturities, freq_discount, sigma, dt, T)
end

function [err, cap_floor_val, cap_floor_val_quote] = optim_fun(params, ycrv_rates, ycrv_maturities, quote_ivol, quote_R, cap_floor_maturities, tenor_LIBOR, freq_discount, caps_floors)

sigma = params;

for cap_floor = 1:numel(quote_ivol) % projdeme vsechny kotace capu / flooru v datasetu
    cap_floor_val(cap_floor) = BDT_cap_floor(quote_R(cap_floor), cap_floor_maturities(cap_floor), tenor_LIBOR, sigma, caps_floors, ycrv_rates, ycrv_maturities, freq_discount);
    cap_floor_val_quote(cap_floor) = black_cap_floor(quote_R(cap_floor), cap_floor_maturities(cap_floor), tenor_LIBOR, quote_ivol(cap_floor), caps_floors, ycrv_rates, ycrv_maturities, freq_discount);
end

% vypocet celkove relativni odchylky cen capu / flooru z HW modelu od cen z trhu
diffs_relative = cap_floor_val./cap_floor_val_quote-1;
err = sum(abs(diffs_relative));
end