function price = BDT_cap_floor(R, T, tenor_LIBOR, sigma, cap_floor, ycrv_rates, ycrv_maturities, freq_discount)
% funkce ziskava hodnotu capu / flooru v BDT modelu
% cap/floor je (xM - yY), tj. cap/floor na sazbu, co se fixuje na x-mesicni LIBOR sazbu a kryje obdobi y let
% model nejdriv spocita hodnoty capletu/floorletu kryjici jednotliva jednoperiodicka obdobi a ty pak vysumuje
% tyto hodnoty se pak vysumuji = cena capu / flooru

%% vstupy
% R                 = cap rate nebo floor rate. Musi byt definovan!!
% T                 = cas konce capu/flooru, napr 5, pokud resime 3M x 5Y deal
% tenor_LIBOR       = tenor LIBOR sazby (v letech), na ktery se vztahuje cap/floor. Nejcasteji 3M = 0.5, 6M = 0.5
% sigma             = parametr volatility kratke sazby r(t) v HW modelu
% cap_floor         = rozliseni, zda cheme pocitat cap (cap_floor = 1) nebo floor (cap_floor = -1)
% ycrv_rates        = vektor urokovych sazeb pozorovanych na trhu
% ycrv_maturities   = splatnosti pro ycrv_rates
% freq_discount     = frekvence uroceni pro spocitani diskontnich faktoru z vynosove krivky ycrv_rates

% prvni reset (a vyplata) je v case t = dt, posledni reset (a vyplata) je jedno odbodbi pred koncem capu/flooru, tedy v case t = T-dt
% caplet/floorlet (i) se restuje v case Ti na LIBOR sazbu L(t, Ti, Ti+dt) = r(Ti, Ti+dt) a vyplaci v case Ti: B(Ti, Ti+dt)*dt*max[w*(r(Ti, Ti+dt) - R), 0], kde w = cap/floor

%% vypocet
reset_and_payoff_times = tenor_LIBOR:tenor_LIBOR:T-tenor_LIBOR; % kalendar resetu a zaroven i vyplat capu/flooru

% stromy BDT od casu t = 0 az do casu posledniho resetu (a vyplaty) t = T-tenor_LIBOR
[BDT_r, ~, BDT_Q, BDT_d, ~, ~] = BDT(ycrv_rates, ycrv_maturities, freq_discount, sigma, tenor_LIBOR, T-tenor_LIBOR);

% ocenujeme jednolive caplety/floorlety indexovane (i)
for i = 1:numel(reset_and_payoff_times)
    idx = BDT_d(:,i+1)>0; % indexy j(i) diskontnich faktoru pro diskont d_ij, tj. hodnot bondu B(Ti, Ti+dt)
    payoff = tenor_LIBOR*BDT_d(idx,i+1).*max(cap_floor*(BDT_r(idx,i+1) - R),0); % vyplata capletu/floorletu, kterou zdiskontujeme jakoby k casu resetu Ti
    values = payoff.*BDT_Q(idx,i+1); % pomoci Arrow-Debreu hodnot Q zjistime soucasnou, psti. vazenou hodnotu (k casu t = 0) techto vyplat capletu/floorletu v jednotlivych bodech stromu
    caplet_floorlet(i) = sum(values); % hodnota i-teho capletu/floorletu je sumaci vsech techto psi. vazenych bodu (k casu t = 0)
end

% hodnota capu/flooru je sumaci vsech capletu/floorletu
price = sum(caplet_floorlet);

%% funkce primo vyuziva funkci
% BDT(ycrv_rates, ycrv_maturities, freq_discount, sigma, dt, T)

%% funkce neprimo vyuziva funkci
% get_r(T, ycrv_rates, ycrv_maturities, freq_discount_in, freq_discount_out)
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
end