function price = BDT_swaption(K, swaption_maturity, IRS_tenor,sigma, ycrv_rates, ycrv_maturities, yearfrac_fix, w_receiver_payer, dt)
% funkce pocita hodnotu swapce v BDT IR modelu

%% vstupy
% K                 = strike swapce
% swaption_maturity = cas T0, tedy cas, kdy swapce expiruje a zaroven zacina bezet IRS v podkladu swapce (T0 je prvni reset floatove nohy IRS)
% IRS_tenor         = delka IRS v podkladu opce. Pozor, toto NENI CAS, kdy IRS konci, je to delka IRS
% sigma             = parametr volatility kratke sazby r(t) v BDT modelu
% ycrv_rates        = vektor urokovych sazeb pozorovanych na trhu
% ycrv_maturities   = splatnosti pro ycrv_rates
% yearfrac_fix      = delka obdobi pro uroceni. Zde ma dve role: 
%                         a) pouziva se pro vypocteni ceny bondu (pro diskontovani timto bondem v opcnim modelu)
%                         b) zaroven se vyuziva i jako yearfrac fixni nohy IRS v podkladu
%                     pozn: na Bloombergu je typicky u swapci i kotovanych sazbe predpokladano S/A uroceni, tj. yearfrac_fix by byl 0.5 
% w_receiver_payer  = 1: hodnota receiver swapce
%                    -1: hodnota payer swapce
% dt                = casovy interval v BDT modelu. dt by melo byt takove, aby odpovidalo tenoru 'kratke' sazby, pro kterou model kalibrujeme (typicky napr. dt = 0.25)

% POZN 1: pokud ocenujeme swapci T0 x (Tn - T0), tak: swaption_maturity = T0, IRS_tenor = Tn - T0 
% POZN 2: tenor (napr 3M, 6M) floatove nohy podkladoveho IRS  je irelevantni, nikam nevstupuje 

%% vypocet
fix_payments_T = swaption_maturity+yearfrac_fix:yearfrac_fix:swaption_maturity+IRS_tenor;
for i = 1:numel(fix_payments_T)
   Ti = fix_payments_T(i);
   [BDT_r, BDT_B, BDT_Q, BDT_d, i_grid, ~] = BDT(ycrv_rates, ycrv_maturities, 1/yearfrac_fix, sigma, dt, Ti);
   t_axis_BDT = i_grid*dt;
   i_crit = find(t_axis_BDT == swaption_maturity);
   BDT_B_store(:,i) = BDT_B(BDT_d(:, i_crit)>0, i_crit);
end

[~, ~, BDT_Q, ~, ~, ~] = BDT(ycrv_rates, ycrv_maturities, 1/yearfrac_fix, sigma, dt, swaption_maturity);

payoffs = max(w_receiver_payer*(K*yearfrac_fix*sum(BDT_B_store, 2) - 1 + BDT_B_store(:,end)), 0);
Q = BDT_Q(BDT_Q(:, end)>0, end);
price = sum(Q.*payoffs);

%% funkce primo vyuziva funkci 
% BDT(ycrv_rates, ycrv_maturities, freq_discount, sigma, dt, T)

%% funkce neprimo vyuziva funkci 
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
% get_r(T, ycrv_rates, ycrv_maturities, freq_discount_in, freq_discount_out)
end
