function price = HW_bond_option(T, s, K, ycrv_rates, ycrv_maturities, freq_discount, kappa, sigma, call_put)
% funkce pocita hodnotu opce (v Hull-Whitove modelu) maturtujici v case T na zero-bond s maturitou v case s>T. 

%% vstupy
% T                 = splatnost opce
% s                 = splatnost dluhopisu v podkladu opce
% K                 = realizacni cena opce
% ycrv_rates        = vektor urokovych sazeb pozorovanych na trhu
% ycrv_maturities   = splatnosti pro ycrv_rates
% freq_discount     = frekvence uroceni. Vyuziva se pro spocitani diskontni faktoru ze sazeb na krivce ycrv_rates
% kappa             = sila mean-reversion v HW modelu
% sigma             = parametr volatility krate sazby r(t) v HW modelu
% call_put          = rozlisteni, zda chceme call opci (call_put = 1) nebo put opci (call_put <> 1)

%% vypocet
v = (sigma^2)/(2*kappa^3) * (exp(kappa*T) - exp(kappa*s)).^2 .* (exp(-2*kappa*s) - exp(-2*kappa*(s+T)));

B_T = get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount);
B_s = get_bond_price(s, ycrv_rates, ycrv_maturities, freq_discount);

d1 = (log(B_s./(K*B_T)) + 0.5*v)./sqrt(v);
d2 = (log(B_s./(K*B_T)) - 0.5*v)./sqrt(v);

if (call_put == 1)
    price = B_s.*normcdf(d1) - K.*B_T.*normcdf(d2);
else
    price = K*B_T.*normcdf(-d2) - B_s.*normcdf(-d1);
end

%% funkce primo vyuziva funkci 
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
end