function price = HW_bond_reconstitution(t, T, x, kappa, sigma, ycrv_rates, ycrv_maturities, freq_discount)
% funkce ziskava cenu bondu B(t,T) v HW1F modelu, pokud je znama stavova velicina x(t) = r(t)-f(0,t)
% (dle definice je vzdy r(0) = f(0,0) a proto x(0) = 0) -> B(0,T) = B(0,T)...
% jedna se o vzorec z Piterbarga II, z proposition 10.1.7

%% vstupy
% t = cas oceneni bondu B(t,T), jehoz hodnotu chceme spocitat
% T = cas splatnosti bondu B(t,T), jehoz hodnotu chceme spocitat
% x = hodnota stavove veliciny x(t) = r(t) - f(0,t)
% kappa             = sila mean-reversion v HW modelu
% sigma             = parametr volatility krate sazby r(t) v HW modelu
% ycrv_rates        = vektor urokovych sazeb pozorovanych na trhu
% ycrv_maturities   = splatnosti pro ycrv_rates
% freq_discount     = frekvence uroceni. Vyuziva se pro spocitani diskontnich faktoru ze sazeb na krivce ycrv_rates

%% vypocet
y = sigma^2*(1-exp(-2*kappa*t))/(2*kappa);
G = (1-exp(-kappa*(T-t)))/kappa;

B0t = get_bond_price(t, ycrv_rates, ycrv_maturities, freq_discount);
B0T = get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount);

B0T = repmat(B0T(:)',numel(x), 1);
G = repmat(G(:)', numel(x), 1);
x = repmat(x(:), 1, numel(T));

price = B0T./B0t.*exp(-x.*G - 0.5*y.*G.^2);
%% funkce primo vyuziva funkci 
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
end