function [kappa, sigma] = HW_calib_to_swaptions(ycrv_rates, ycrv_maturities, quote_ivol, quote_K, swaption_maturities, IRS_tenors, yearfrac_fix)
% funkce kalibruje HW IR model na trzni kotace evropskych swapci
% vystupem jsou parametry kappa, sigma HW modelu

% optimalizace pracuje s hledanim nejlepsiho fittu na ceny payer swapci, ktere ziskame Blackovym swapcnim modelem 
% nejlepsi fit je ve smyslu nejmensi celkove relativni chyby, tedy suma( abs(cena_HW_model(i)/cena_Black(i) - 1) ), pres vsechna i. Kazde i je jedna swapce z vektoru (quote_ivol a quote_K)

%% vstupy
% ycrv_rates            = vektor urokovych sazeb pozorovanych na trhu
% ycrv_maturities       = splatnosti pro ycrv_rates
% quote_ivol            = kotace blackovych implikovanych volatilit pro swapce. (vektor)
% quote_K               = striky swapci. (vektor)
% swaption_maturities   = expirace kotovanych swapci (v letech) 
% IRS_tenors            = tenory podkladovych IRS
% yearfrac_fix          = jakoby tenor fixni nohy IRS. Zaroven se pouziva tento yearfrac (-> frekvence uroceni) i pro spocitani diskontu ve vsech vzorcich

%% vypocet
f = @(params)(optim_fun(params, ycrv_rates, ycrv_maturities, quote_ivol, quote_K, swaption_maturities, IRS_tenors, yearfrac_fix));
params = fmincon(f, [0.3 0.02], [], [], [], [], [-0.1 0.000001], [2 0.5]);

[err, swaption_val, swaption_val_quote] = optim_fun(params, ycrv_rates, ycrv_maturities, quote_ivol, quote_K, swaption_maturities, IRS_tenors, yearfrac_fix);

kappa = params(1);
sigma = params(2);

scatter(swaption_val_quote*10000, swaption_val*10000); axis([0 150 0 150]); xlabel('swaption premia - market (bps)'); ylabel('swaption premia - HW model (bps)'); grid on;
set(gca,'XTick',0:30:150);
set(gca,'YTick',0:30:150);
hold on;
plot(0:150, 0:150, 'r--');
hold off;
%% funkce primo vyuziva funkci 
% optim_fun(params, ycrv_rates, ycrv_maturities, quote_ivol, quote_K, swaption_maturities, IRS_tenors, yearfrac_fix) ... zde v m. filu
% HW_swaption_price(K, swaption_maturity, IRS_tenor, kappa, sigma, ycrv_rates, ycrv_maturities, yearfrac_fix, w_receiver_payer)
% black_swaption(K, swaption_maturity, IRS_tenor, sigma, yearfrac_fix, w_receiver_payer, ycrv_rates, ycrv_maturities)

%% funkce neprimo vyuziva funkci 
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
% HW_bond_option(T, s, K, ycrv_rates, ycrv_maturities, freq_discount, kappa, sigma, call_put)
% HW_bond_reconstitution(t, T, x, kappa, sigma, ycrv_rates, ycrv_maturities, freq_discount)
end

function [err, swaption_val, swaption_val_quote] = optim_fun(params, ycrv_rates, ycrv_maturities, quote_ivol, quote_K, swaption_maturities, IRS_tenors, yearfrac_fix)
% objektivni funkce pro vypocet celkove relativni chyby.
% relativni chybou se rozumi odchylka cen swapci z HW modelu a kotovanych cen (tj. z Blackova modelu)

w_receiver_payer = -1; % nastavujeme, ze hledame cenu payer swapce
kappa = params(1); % mean reversion speed HW
sigma = params(2); % sigma v HW modelu

for swaption = 1:numel(quote_ivol) % projdeme vsechny swapce v datasetu
    swaption_val(swaption) = HW_swaption_price(quote_K(swaption), swaption_maturities(swaption), IRS_tenors(swaption), kappa, sigma, ycrv_rates, ycrv_maturities, yearfrac_fix, w_receiver_payer);
    swaption_val_quote(swaption) = black_swaption(quote_K(swaption), swaption_maturities(swaption), IRS_tenors(swaption),quote_ivol(swaption), yearfrac_fix, w_receiver_payer, ycrv_rates, ycrv_maturities);
end

% vypocet celkove relativni odchylky cen swapci z HW modelu od cen z trhu
diffs_relative = swaption_val./swaption_val_quote-1;
err = sum(abs(diffs_relative));
%err = sum(abs(swaption_val - swaption_val_quote)*10000);
end