function price = HW_cap_floor_price(R, T, tenor_LIBOR, kappa, sigma, cap_floor, ycrv_rates, ycrv_maturities, freq_discount)
% funkce ziskava hodnotu capu / flooru v Hull-Whitove modelu
% cap = porfolio put opci na zero bond s mod strikem, floor = portfolio opci na zero bond s mod strikem
% princip tedy spociva ve vypoctu mod striku R_mod, a volani funkce pro bondove opce v HW modelu (HW_bond_option) / R_mod
% tyto hodnoty se pak vysumuji = cena capu / flooru

%% vstupy
% R                 = cap rate nebo floor rate. Musi byt definovan!!
% T                 = cas konce capu/flooru, napr 5, pokud resime 3M x 5Y deal
% tenor_LIBOR       = tenor LIBOR sazby (v letech), na ktery se vztahuje cap/floor. Nejcasteji 3M = 0.5, 6M = 0.5
% kappa             = sila mean-reversion v HW modelu
% sigma             = parametr volatility kratke sazby r(t) v HW modelu
% cap_floor         = rozliseni, zda cheme pocitat cap (cap_floor = 1) nebo floor (cap_floor <> 1)
% ycrv_rates        = vektor urokovych sazeb pozorovanych na trhu
% ycrv_maturities   = splatnosti pro ycrv_rates
% freq_discount     = frekvence uroceni pro spocitani diskontnich faktoru z vynosove krivky ycrv_rates

% zakladem jsou casy Ti = i*tenor_LIBOR, kdy se resetuje a vyplaci
% posledni reset je jedno odbobi pred koncem capu/flooru T
% mame tedy casy resetu reset_dates
reset_dates = tenor_LIBOR : tenor_LIBOR : T - tenor_LIBOR;

R_mod = 1/(1+tenor_LIBOR*R);

if (cap_floor == 1) % cap je portfolio put opci na zero bondy
    call_put = 0;
else % floor je portfolio call opci na zero bondy
    call_put = 1;
end

caplets_floorlets = 1/R_mod*HW_bond_option(reset_dates, reset_dates + tenor_LIBOR, R_mod, ycrv_rates, ycrv_maturities, freq_discount, kappa, sigma, call_put);
price = sum(caplets_floorlets);

%% funkce primo vyuziva funkci 
% HW_bond_option(T, s, K, ycrv_rates, ycrv_maturities, freq_discount, kappa, sigma, call_put)

% funkce primo vyuziva funkci 
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
end