function [price, R_out] = black_cap_floor(R, T, tenor_LIBOR, sigma, cap_floor, ycrv_rates, ycrv_maturities, freq_discount)
% funkce pocita hodnotu capu/flooru, ktery ihned zacina, ma cap/floor sazbu R a kryje obdobi [0+tenor_LIBOR, T]
% vraci i hodnotu cap / floor rate, protoze tu je potreba v ATM pripadech dopocitavat, pokud neni tato hodnota R znama a jen vime 'ATM'
% pocita se tedy s tim, ze pokud mame napr. deal 3M x 5Y, tak prvni reset a vyplata na zaklade tohoto resetu je v case 3M
% posledni reset a vyplata je v case: (T-tenor_LIBOR), v tomto pripade v case 4Y + 9M
% pokud tedy celkove ma obodbi n = cas_konce_capletu_floorlet/tenor_LIBOR obodbi, potom je jen (n-1) vyplat capu/flooru
% funkce si umi sama dopocitat ATM cap / floor rate R, pokud ho nezname, ale vime, ze kotace volatility je pro ATM cap / floor

% funkce tez umi pracovat vektorove, pokud T, sigma a R jsou vektory. V takovem pripade vraci hodnotu capu / flooru pro ruzne maturity 

%% vstupy
% R                 = cap rate nebo floor rate. Pokud je definovan jako ATM, <= 0, nebo prazdna hodnota [], tak se ATM cap/floor rate automaticky dopocita a pouzije se pak dale
% T                 = cas konce capu/flooru, napr 5, pokud resime 3M x 5Y deal. Muze byt i vektor
% tenor_LIBOR       = tenor LIBOR sazby (v letech), na ktery se vztahuje cap/floor. Nejcasteji 3M = 0.5, 6M = 0.5
% sigma             = implikovana volatilita pro dany cap/floor a vsechny jeho caplety/floorlety. Muze byt i vektor
% cap_floor         = rozliseni, zda cheme pocitat cap (cap_floor = 1) nebo floor (cap_floor <> 1)
% ycrv_rates        = vektor urokovych sazeb pozorovanych na trhu
% ycrv_maturities   = splatnosti pro ycrv_rates
% freq_discount     = frekvence uroceni pro spocitani diskontnich faktoru z vynosove krivky ycrv_rates

%% vypocet

for i = 1: numel(sigma)
% zakladem je, ze jsou casy Ti = i*tenor_LIBOR, kdy se resetuje a vyplaci
% posledni reset je jedno odbobi pred koncem capu/flooru T
% mame tedy casy resetu reset_dates
    reset_dates = tenor_LIBOR: tenor_LIBOR : T(i) - tenor_LIBOR;

    if (strcmp(R(i), 'ATM') || isempty(R) || R(i) <= 0) % pripad ATM kotaci, kdy nezname cap / floor rate R
        % ATM rate R znamena, ze cap(R) stoji stejne jako stejny floor(R)
        % musime tedy numericky dopocitat R
        f = @(R) ( sum(black_caplet_floorlet(R, reset_dates, tenor_LIBOR, sigma(i), 1, ycrv_rates, ycrv_maturities, freq_discount)) ...
                     - sum(black_caplet_floorlet(R, reset_dates, tenor_LIBOR, sigma(i), 0, ycrv_rates, ycrv_maturities, freq_discount)) );
        R_out(i) = fzero(f, [0.0001 0.2]);
    else
        R_out(i) = R(i);
    end

    caplets_floorlets = black_caplet_floorlet(R_out(i), reset_dates, tenor_LIBOR, sigma(i), cap_floor, ycrv_rates, ycrv_maturities, freq_discount);
    price(i) = sum(caplets_floorlets);
end
price = price(:);
R_out = R_out(:);
%% funkce primo vyuziva funkci
% black_caplet_floorlet(R, T, tenor_LIBOR, sigma, caplet_floorlet, ycrv_rates, ycrv_maturities, freq_discount)

%% funkce neprimo vyuziva funkci
% get_LIBOR_rate(T, tenor_LIBOR, ycrv_rates, ycrv_maturities, freq_discount)
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
end