function price = black_caplet_floorlet(R, T, tenor_LIBOR, sigma, caplet_floorlet, ycrv_rates, ycrv_maturities, freq_discount)
% funkce pocita hodnotu capletu/floorletu, ktery ma cap/floor sazbu R a kryje obdobi [T, T + tenor_LIBOR]
% funkce vse pocita primo z pozorovane vynosove krivky (a kotace sigma). Neni treba dodavat fwd LIBOR sazbu..

%% vstupy
% R                 = cap rate nebo floor rate
% T                 = splatnost capletu/floorletu. Zaroven je to cas, kdy caplet/floorlet vyplaci (jednoperiodicky diskonotvanou) hodnotu
% tenor_LIBOR       = tenor LIBOR sazby (v letech), na ktery se vztahuje caplet/floorlet. Nejcasteji 3M = 0.5, 6M = 0.5
% sigma             = implikovana volatilita pro dany caplet/floorlet
% caplet_floorlet   = rozliseni, zda cheme pocitat caplet (caplet_floorlet = 1) nebo floorlet (caplet_floorlet <> 1)
% ycrv_rates        = vektor urokovych sazeb pozorovanych na trhu
% ycrv_maturities   = splatnosti pro ycrv_rates
% freq_discount     = frekvence uroceni pro spocitani diskontnich faktoru z vynosove krivky ycrv_rates

%% vypocet
L = get_LIBOR_rate(T, tenor_LIBOR, ycrv_rates, ycrv_maturities, freq_discount); % spocitame forwardovou sazbu L(t,T,T+tenor_LIBOR)
d1 = (log(L./R) + 0.5*sigma.^2*T)./(sigma.*sqrt(T));
d2 = d1 - sigma.*sqrt(T);

BtT_plus_tenor = get_bond_price(T + tenor_LIBOR, ycrv_rates, ycrv_maturities, freq_discount);

if (caplet_floorlet == 1) % pocitame hodnotu capletu
    price = tenor_LIBOR.*BtT_plus_tenor.*(L.* normcdf(d1) - R.*normcdf(d2));
else % pocitame hodnotu floorletu
    price = tenor_LIBOR.*BtT_plus_tenor.*(R.*normcdf(-d2) - L.* normcdf(-d1));
end

%% funkce primo vyuziva funkci
% get_LIBOR_rate(T, tenor_LIBOR, ycrv_rates, ycrv_maturities, freq_discount)
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
end