function price = black_swaption(K, swaption_maturity, IRS_tenor, sigma, yearfrac_fix, w_receiver_payer, ycrv_rates, ycrv_maturities)
% implementace Blackova vzorce pro swapce. Vystupem je cena swapce

%% vstupy
% K                 = strike swapce
% swaption_maturity = cas T0, tedy cas, kdy swapce expiruje a zaroven zacina bezet IRS v podkladu swapce (T0 je prvni reset floatove nohy IRS)
% IRS_tenor         = delka IRS v podkladu opce. Pozor, toto NENI CAS, kdy IRS konci, je to DELKA IRS
% sigma             = implikovana volatilita pro swapci
% yearfrac_fix      = delka obdobi pro uroceni. Zde ma dve role: 
%                         a) pouziva se pro vypocteni ceny bondu, kterymi se diskontuje vyplata swapce. Tj. k ziskani frekvence uroceni pro spocitani cen bondu
%                         b) zaroven se vyuziva i jako yearfrac fixni nohy IRS v podkladu (to je videt na vzorci pro annuitu nize)
%                     pozn: na Bloombergu je typicky u swapci i kotovanych sazbe predpokladano S/A uroceni, tj. yearfrac_fix by byl 0.5 
% w_receiver_payer  = 1: hodnota receiver swapce
%                    -1: hodnota payer swapce
% ycrv_rates        = vektor urokovych sazeb pozorovanych na trhu
% ycrv_maturities   = splatnosti pro ycrv_rates

% POZN: pokud ocenujeme swapci T0 x (Tn - T0), tak: swaption_maturity = T0, IRS_tenor = Tn - T0 
% POZN 2: tenor (napr 3M, 6M) floatove nohy podkladoveho IRS  je irelevantni, nikam nevstupuje 

%% vypocet
R = get_fwd_swap_rate(swaption_maturity, IRS_tenor, ycrv_rates, ycrv_maturities, 1/yearfrac_fix);

I = IRS_tenor/yearfrac_fix;
i = 1:I;
B = get_bond_price(swaption_maturity + i*yearfrac_fix, ycrv_rates, ycrv_maturities, 1/yearfrac_fix);
annuity = yearfrac_fix*sum(B);

d1 = (log(R/K) + (0.5*sigma*sigma*swaption_maturity))/(sigma*sqrt(swaption_maturity));
d2 = d1 - sigma*sqrt(swaption_maturity);

w = w_receiver_payer;
price = (w.*R*normcdf(w*d1) - w.*K*normcdf(w.*d2))*annuity;

%% funkce primo vyuziva funkci 
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
% get_fwd_swap_rate(IRS_start_T, IRS_tenor, ycrv_rates, ycrv_maturities, freq_discount)
end