function L = get_LIBOR_rate(T, tenor_LIBOR, ycrv_rates, ycrv_maturities, freq_discount)
% funkce ziskava forwardovou LIBOR sazbu L(t,T) = L(t,T,T+tenor_LIBOR). Tedy sazbu, ktera se resetuje v case T a vyplaci v case T+tenor_LIBOR
% hodnota teto sazby L(t,T) je definovana vztahem L(t,T) = 1/delta*( B(t,T)/B(t,T+delta) - 1)

%% vstupy
% T                 = resety LIBOR sazby L(t,T). T musi byt > t
% tenor_LIBOR       = tenor LIBOR sazby (v letech). Vetsinou 3M = 0.25, 6M = 0.5 atd.
% ycrv_rates        = vektor urokovych sazeb pozorovanych na trhu
% ycrv_maturities   = splatnosti pro ycrv_rates
% freq_discount     = frekvence uroceni pro spocitani diskontnich faktoru z vynosove krivky ycrv_rates

BtT = get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount);
BtT_tenor_LIBOR = get_bond_price(T+tenor_LIBOR, ycrv_rates, ycrv_maturities, freq_discount);

L = 1./tenor_LIBOR.*(BtT./BtT_tenor_LIBOR -1);

%% funkce primo vyuziva funkci 
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
end