function price = get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
% funkce pocita cenu bondu B(0,T) pomoci sazeb na vynosove krive (ycrv_rates a ycrv_maturities)
% funkce rozlisuje frekvenci uroceni pro vypocet diskontu
% interpoluje diskontni faktor

%% vstupy
% T                 = splatnost/i, pro ktere pocitame ceny bondu
% ycrv_rates        =   (a) vektor urokovych sazeb vynosove krivky
%                       (b) jedna jedina sazba - v takovem pripade se nebude interpolovat a vsechny ceny bondu se spocitaji z teto sazby
% yrcv_maturities   = splatnost nebo splatnosti pro ycrv_rates
% freq_discount     = frekvence uroceni pro spocteni diskontu. Pokud je freq = inf nebo freq = [], tak se pouziva spojite uroceni

%% vypocet
if numel(ycrv_rates > 1) % pripad, kdy je ycrv_rates vektor (ne jen jedna sazba)
    if (iscolumn(ycrv_rates) && (ismember(0, ycrv_maturities) == 0))
        ycrv_rates = [ycrv_rates(1); ycrv_rates];
        ycrv_maturities = [0; ycrv_maturities];
    end
    if (isrow(ycrv_rates) && (ismember(0, ycrv_maturities) == 0))
        ycrv_rates = [ycrv_rates(1) ycrv_rates];
        ycrv_maturities = [0 ycrv_maturities];
    end

    if (isinf(freq_discount) || isempty(freq_discount) )
        bond_prices = exp(-ycrv_rates.*ycrv_maturities);
    else
        bond_prices = 1./(1+ycrv_rates/freq_discount).^(freq_discount*ycrv_maturities);
    end

    price = interp1(ycrv_maturities,bond_prices,T, 'pchip');
else % pripad, kdy je ycrv_rates jen jedna sazba
    if (isinf(freq_discount) || isempty(freq_discount) )
        bond_prices = exp(-ycrv_rates.*ycrv_maturities);
    else
        bond_prices = 1./(1+ycrv_rates/freq_discount).^(freq_discount*ycrv_maturities);
    end
    price = bond_prices;
end
%% funkce primo vyuziva funkci 
% -
end