function f = get_continuous_fwd_rate(T, dT, ycrv_rates, ycrv_maturities, freq_discount)
% funkce numericky ziskava okamzitou forwardovou sazbu f(0,T) = f(0,T,T+dT) z pozorovane spotove krivky na trhu.
% funkce vyuziva vztahu: f(t,T) = -dln(B(t,T))/dT
% funkce umi pracovat i vektorove, tj. pro ruzny rozsah T (muze to byt vektor)

%% vstupy
% T = splatnost nebo splatnosti, pro ktere chceme pocitat okamzitou forwardovou sazbu f(0,T)
% dT = maly posun casu pro spocitani derivace dln(B(t,T))/dT = ( ln(B(t,T+dT)) - ln(B(t,T)) )/dT
% ycrv_rates        = vektor urokovych sazeb pozorovanych na trhu
% ycrv_maturities   = splatnosti pro ycrv_rates
% freq_discount     = frekvence uroceni. Vyuziva se pro spocitani diskontni faktoru ze sazeb na krivce ycrv_rates

%% vypocet
bond_value_T = get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount);
bond_value_T_plus_dT = get_bond_price(T+dT, ycrv_rates, ycrv_maturities, freq_discount);

dlnB = log(bond_value_T_plus_dT) - log(bond_value_T);

f = -dlnB/dT;

%% funkce primo vyuziva funkci 
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
end