function R = get_fwd_swap_rate(IRS_start_T, IRS_tenor, ycrv_rates, ycrv_maturities, freq_discount)
% funkce pocita forwardovou swapovou sazbu R(t) as of t = 0. Tato je potreba pro ocenovani swapci v Blackove modelu
% R(t) je takova 'fer' sazba za kterou kdybychom nyni uzavreli IRS startujici v case T0 = IRS_start_T a bezici do casu T0 + IRS_tenor, tak tento IRS bude mit hodnotu 0

%% vstupy
% IRS_start_T = cas T0, kdy startuje IRS. V tomto case taky bude prvni refix floatove nohy IRS
% IRS_tenor  = delka IRS, pro ktery chceme pocitat R(t). Pozor, toto NENI CAS, kdy IRS konci, je to DELKA IRS
% ycrv_rates        = vektor urokovych sazeb pozorovanych na trhu
% ycrv_maturities   = splatnosti pro ycrv_rates
% freq_discount     = parametr frekvence uroceni :
%                           a) vyuziva se pro spocitani diskontni faktoru ze sazeb na krivce ycrv_rates
%                           b) take se zde vyuziva pro spocteni delky obdobi, kdy fixni noha IRS vyplaci urok, tj. yearfrac_fixed_leg = 1/freq_discount

% POZN : tenor (napr 3M, 6M) floatove nohy podkladoveho IRS je pro vypocet R(t) irelevantni, nikam nevstupuje 

%% vypocet
yearfrac_fixed_leg = 1/freq_discount;
I = IRS_tenor/yearfrac_fixed_leg;

% denominator (v podstate odpovida PV CF z fixni nohy)
i = 1:I;
B = get_bond_price(IRS_start_T + i*yearfrac_fixed_leg, ycrv_rates, ycrv_maturities, freq_discount);
denom = yearfrac_fixed_leg*sum(B);

% numer (v podstate odpovida PV CF z floatove nohy)
B_T0 = get_bond_price(IRS_start_T, ycrv_rates, ycrv_maturities, freq_discount);
numer = (B_T0 - B(end));

R = numer/denom;

%% funkce primo vyuziva funkci 
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
end