function r = get_r(T, ycrv_rates, ycrv_maturities, freq_discount_in, freq_discount_out)
% funkce ziskava spotovou urokovou sazbu r = r(0,T) pro tenor(y) T, pokud zadame vstupni vynosovou krivku
% funkce take umi provest konverzi sazeb dle frekvence uroceni (napr. prvede sazby ze spojiteho uroceni na slozene). 
% to kontroluji parametry freq_discount_in a freq_discount_out
% funkce neinterpoluje sazby, ale diskontni faktory

%% vstupy
% T                 = splatnosti, pro ktere chceme ziskat spotove sazby r(0, T)
% ycrv_rates        = vektor urokovych sazeb pozorovanych na trhu pri frekvenci freq_discount_in
% ycrv_maturities   = splatnosti pro ycrv_rates
% freq_discount_in  = frekvence uroceni pro spocitani diskontnich faktoru z vynosove krivky ycrv_rates
% freq_discount_out = frekvence uroceni, pro vystupni sazby

%% vypocet
if (max(T)>max(ycrv_maturities)); warning('pozadujeme sazbu pro vyssi tenor nez je max tenor u vstupni ycrv. Bylo by dobre doplnit dalsi sazby do ycrv rates a maturities'); end;
bond_prices = get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount_in);
if (isinf(freq_discount_out))
    r = -log(bond_prices)./T;
else
    r = freq_discount_out*( (1./bond_prices).^(1./(freq_discount_out*T)) -1.0 );
end

%% funkce primo vyuziva funkci 
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
end