function price = geske_compound(S, r, q, sigma, K1, K2, T1, T2, w1, w2)
% implmentace Geskeho Compound opce (opce na opci) podle http://investexcel.net/compound-options-excel/ (pozor, je tam chyba! V b1 a b2 ma byt T2 misto T1!)
% opce o1(K1, T1) na opci o2 (s K2, T2), tedy
% o1 - typ 'hlavni' opce v nazvu; 1 = call; 0 = put. Tato opce maturuje v T1
% o2 - typ 'male' opce v nazvu;   1 = call; 0 = put. Tato opce maturuje v T2. Je to vlastne podklad
% hlavni opce ma T1 a K1; podrizena opce ma tedy T2 a K2

%% vstupy
% S  = spotova cena podkladoveho aktiva
% r  = hodnota risk-free sazby
% q  = hodnota spojite dividendy podkladoveho aktiva S
% K1 = strike opce o1, ktera maturuje v T1
% T1 = residualni doba do splatnosti opce o1, se strikem K1
% K2 = strike opce o2, ktera maturuje v T2
% T2 = residualni doba do splatnosti opce o2, se strikem K2
% w1 = typ 'hlavni' opce o1 (1 = call, -1 = put)
% w2 = typ 'vedlejsi' opce o2 (1 = call, -1 = put)

% POZN: kombinace napr. w1 = -1, w2 = 1 rika: put opce (maturujici v case T1 se strikem K1) na call opci (maturujici v case T2 se strikem K2)

%% vypocet
% S_crit = kriticka hodnota akcie, pro kterou bude platit o2(S_crit,K2, T1,T2) = K1
f = @(S)(black_scholes(S, K2, r, q, T2 - T1, sigma, w2) - K1);
S_crit = fzero(f, [0.01 10000]);

a1 = (log(S/S_crit) + (r-q + 0.5*sigma^2)*T1)/(sigma*sqrt(T1));
a2 = a1 - sigma*sqrt(T1);
b1 = (log(S/K2) + (r-q + 0.5*sigma^2)*T2)/(sigma*sqrt(T2));
b2 = b1 - sigma*sqrt(T2);
rho = sqrt(T1/T2);

price = w1*w2*S*exp(-q*T2)*mvncdf([w1*w2*a1 w2*b1], [0 0], [1 w1*rho; w1*rho 1]) - w1*w2*K2*exp(-r*T2)*mvncdf([w1*w2*a2 w2*b2], [0 0], [1 w1*rho; w1*rho 1]) - w1*exp(-r*T1)*K1*normcdf(w1*w2*a2);

% fukcni, ale krkololmna implementace
% SIGMA = [1 rho; rho 1];
% SIGMAneg = [1 -rho; -rho 1];
% if ((w1 == 1) && (w2 == 1)) % call on call
%     price = S*exp(-q*T2)*mvncdf([a1 b1], [0 0], SIGMA) - K2*exp(-r*T2)*mvncdf([a2 b2], [0 0], SIGMA) - exp(-r*T1)*K1*normcdf(a2);
% elseif ((w1 == 1) && (w2 ~= 1)) % call on put
%     price = K2*exp(-r*T2)*mvncdf([-a2 -b2], [0 0], SIGMA) - S*exp(-q*T2)*mvncdf([-a1 -b1], [0 0], SIGMA) - exp(-r*T1)*K1*normcdf(-a2);
% elseif ((w1 ~= 1) && (w2 == 1)) % put on call
%     price = K2*exp(-r*T2)*mvncdf([-a2 b2], [0 0], SIGMAneg) - S*exp(-q*T2)*mvncdf([-a1 b1], [0 0], SIGMAneg) + exp(-r*T1)*K1*normcdf(-a2);
% elseif ((w1 ~= 1) && (w2 ~= 1)) % put on put
%     price = S*exp(-q*T2)*mvncdf([a1 -b1], [0 0], SIGMAneg) - K2*exp(-r*T2)*mvncdf([a2 -b2], [0 0], SIGMAneg) + exp(-r*T1)*K1*normcdf(a2);
% end

%% funkce primo vyuziva funkci
% black_scholes(S, K, r, q, T, sigma, w)
end
