function [price] = heston_trap(S, K, T, r, q, v, theta, kappa, sigma, rho, w)
% implementace hestonova modelu Rouah Vainberg (s prihlednutim k 'trap' problemu dle Albrechera)
% numericka integrace probiha pomoci matlabovske inbuilt fce quad

%% vstupy
% S         = spotova cena podkladoveho aktiva
% K         = strike opce
% T         = residualni doba do splatnosti opce
% r         = hodnota risk-free sazby pro splatnost T
% q         = hodnota spojite dividendy
% v         = rozptyl v case oceneni (t = 0)
% theta     = dlouhodoba uroven (mean revesion uroven) rozptylu
% kappa     = rychlost mean revesion rozptylu
% sigma     = volatilita rozptylu
% rho       = korelace mezi Wienerovskymi procesy rozptylu a ceny podkladu
% w         = identifikator call opce (w = 1) nebo put opce (w = -1)

%% vypocet
u = [0.5 -0.5];
b = [kappa - rho*sigma kappa];

d = @(j, phi)( sqrt((rho*sigma*phi*1i-b(j)).^2 - sigma^2*(2*u(j)*phi*1i-phi.^2)) );
c = @(j, phi)( (b(j) - rho*sigma*phi*1i-d(j, phi))./(b(j) - rho*sigma*phi*1i+d(j, phi)));
D = @(j, phi)( (b(j) - rho*sigma*phi*1i - d(j, phi))/sigma^2.* (1-exp(-d(j, phi)*T))./(1 - c(j, phi).*exp(-d(j, phi)*T)) );
C = @(j, phi)( (r-q)*phi*1i*T + kappa*theta/sigma^2 * ( (b(j) - rho*sigma*phi*1i - d(j, phi))*T - 2*log((1-c(j, phi).*exp(-d(j, phi)*T))./(1-c(j, phi))) ) );
f1_2 = @(j, phi)(exp(C(j, phi)+D(j, phi)*v+1i*phi*log(S)));

f = @(phi)( real(exp(-1i*phi*log(K))./(1i*phi).*(S*exp(-q*T)*f1_2(1, phi) - K*exp(-r*T)*f1_2(2, phi))) );

result_comb_call = 1/2*S*exp(-q*T) - 1/2*K*exp(-r*T)+ 1/pi*integral(f, 0.0, 1000.0);
result_comb_put = result_comb_call + K*exp(-r*T) - S*exp(-q*T);

price = result_comb_call*(w == 1) + result_comb_put*(w == -1);

%% funkce primo vyuziva funkci
% -
end