function price = zhou_JD_option(S, K, T, r, q, sigma, lambda, mu_Y, sigma_Y, w)
% funkce vraci hodnotu opce v Zhouove JD modelu
% implementovano dle popisu v me disertaci
% kazdy skok nastava s intenzitou lambda, velikost skoku je lognormalne rozelena (Yi +1) ~ ln(mu_Y, sigma_Y^2)

%% vstupy
% S         = spotova cena podkladoveho aktiva
% K         = strike opce
% T         = residualni doba do splatnosti opce
% r         = hodnota risk-free sazby pro splatnost T
% q         = hodnota spojite dividendy
% lambda    = intenzita skoku
% mu_Y      = stredni hodnota nahodne promenne ln(Yi +1), kde Yi je velikost i-teho skoku (nahodna promenna)
% sigma_Y   = smerodatna odchylka nahodne promenne ln(Yi +1), kde Yi je velikost i-teho skoku (nahodna promenna)
% w         = identifikator call opce (w = 1) nebo put opce (-1)

%% vypocet
S = S(:);
dim_S = numel(S);
i = 0:20; % hypoteticky pocket skoku, ktery by mohl nastat. Cislo 20 je dostatence, koeficienty budou exponentialne klesat
beta = exp(mu_Y + 0.5*sigma_Y^2) - 1; % beta = E[Yi], tj. ocekavana hodnota skoku

sigma_i_sq = repmat(sigma^2 *T + i*sigma_Y^2, dim_S, 1);
mu_i = repmat((r-q - 0.5*sigma^2 - lambda*beta)*T + i*mu_Y, dim_S, 1);
d1 = (log(repmat(S, 1, max(i)+1)/K) + mu_i + sigma_i_sq)./sqrt(sigma_i_sq);
d2 = d1 - sqrt(sigma_i_sq);

fact_i = factorial(i);
sum_term_1 = exp(-lambda*T*(beta+1) + i*mu_Y + 0.5*i*sigma_Y^2).*(lambda*T).^i ./ fact_i;
sum_term_2 = exp(-lambda*T).*(lambda*T).^i ./ fact_i;

price = w*S.*exp(-q*T).*sum(repmat(sum_term_1, dim_S, 1).*normcdf(w*d1), 2) - w*K*exp(-r*T)*sum(repmat(sum_term_2, dim_S, 1).*normcdf(w*d2), 2);

%% funkce primo vyuziva funkci
% -
end