function P = PIDE_prob_HiHj(T, m1, m2, n, theta_scheme)
% funkce resi PIDE pro Pr[min[tau_1, tau_2]<=T] dle vztahu na strane 90 a to v ramci CPS modelu
% parametry procesu jump-diffusion intenzit jsou z duvodu prehlednosti definovany uvnitr teto funkce (nize)

% priklad spusteni: P = PIDE_prob_HiHj(5, 80, 80, 40, 0.5);

% parametry spojiteho procesu
gamma1 = 0.026;  kappa1 = 0.05;  theta1 = log(0.04); sigma1 = 0.4;
gamma2 = 0.03;   kappa2 = 0.05;  theta2 = log(0.02); sigma2 = 0.2;
rho = 0.999;

% parametry skok procesu
lambda = 2.0;
a1 = 1; a2 = 1;
J_mu = 0; J_sigma = 1.0; 

x1 = log(gamma1);
x2 = log(gamma2);
x1_floor    = -7.0;
x1_ceil     =  0.5;
x2_floor    = -7.0;
x2_ceil     =  1.5;

dx1 = (x1_ceil-x1_floor)/(m1+1);
x1_grid = x1_floor+dx1:dx1:x1_ceil-dx1;
dx2 = (x2_ceil-x2_floor)/(m2+1);
x2_grid = x2_floor+dx2:dx2:x2_ceil-dx2;

dt = T/n;
t = 0:dt:T;

V = ones(m1*m2,n+1);
x1_grid_rep = repmat(x1_grid(:), 1, m2); x1_grid_rep = x1_grid_rep(:);
x2_grid_rep = repmat(x2_grid(:)',m1, 1); x2_grid_rep = x2_grid_rep(:);
A = get_A(x1_grid, x2_grid, kappa1, theta1, sigma1, kappa2, theta2, sigma2, rho, a1, a2, lambda, J_mu, dx1, dx2);

I = speye(m1*m2);
RHS_tmp = I+dt*(1-theta_scheme)*A;
LHS = I-dt*(1-theta_scheme)*A;

[z,w]=lgwt(50,-2,2);
Gz = normpdf(z, J_mu, J_sigma);
Gxw = repmat(Gz.*w, 1, m1*m2);

x1_grid_rep = repmat(x1_grid(:), 1, m2); x1_grid_rep = x1_grid_rep(:);
x2_grid_rep = repmat(x2_grid(:)',m1, 1); x2_grid_rep = x2_grid_rep(:);

x1_plus_jump_z = a1*repmat(z, 1, numel(x1_grid_rep))+repmat(x1_grid_rep(:)', numel(z), 1);
x2_plus_jump_z = a2*repmat(z, 1, numel(x2_grid_rep))+repmat(x2_grid_rep(:)', numel(z), 1);

for i = numel(t)-1:-1:1
    V_fut = reshape(V(:, i+1), [m1, m2]);
    V_fut_mod = [V_fut(:, 1) V_fut V_fut(:, end)]; V_fut_mod = [V_fut_mod(1,:); V_fut_mod; V_fut_mod(end,:)];
    V_jump = interp2([-10^20 x2_grid 10^20], [-10^20 x1_grid 10^20], V_fut_mod, x2_plus_jump_z, x1_plus_jump_z);

    
    V_fut_mat = repmat(V_fut(:)', numel(w), 1);
    
    V_integral = (V_jump-V_fut_mat).*Gxw;
    V_term = sum(V_integral);
    
    RHS = RHS_tmp*V(:, i+1) + dt*lambda*V_term(:);
    [V(:, i), ~] = bicgstab(LHS, RHS);
end

surf(x2_grid, x1_grid, 1-reshape(V(:, 1), [m1, m2])); xlabel('$x_2 = \ln {\gamma _2}$', 'interpreter', 'latex'); ylabel('$x_1 = \ln {\gamma _1}$', 'interpreter', 'latex'); 
zlabel('Pr$\left[ {\min [\tau _1^*,\tau _2^*] \le t} \right]$', 'interpreter', 'latex'); colormap hsv; %axis tight;

P = interp2(x2_grid, x1_grid, 1-reshape(V(:, 1), [m1, m2]), x2, x1);
end

function A = get_A(x1_grid, x2_grid, kappa1, theta1, sigma1, kappa2, theta2, sigma2, rho, a1, a2, lambda, J_mu, dx1, dx2)

m1 = numel(x1_grid);
m2 = numel(x2_grid);

%A = zeros(m1*m2);

c1_vec = []; c1_col = []; rows_1 =[];
c2_vec = []; c2_col = []; rows_2 = [];
c12_vec = []; c12_col = []; rows_12 = [];

get_idx = @(j1, j2)( (j2 - 1)*m1+j1);
for j1 = 1:m1
   for j2 = 1:m2
       A_row = get_idx(j1, j2);
       
       [coeffs1, coeffs_col1]   = lcu1(j1, j2, x1_grid, x2_grid, kappa1, theta1, sigma1, a1, lambda, J_mu, dx1);
       [coeffs2, coeffs_col2]   = lcu2(j1, j2, x1_grid, x2_grid, kappa2, theta2, sigma2, a2, lambda, J_mu, dx2);
       [coeffs12, coeffs_col12] = lcu12(j1, j2, x1_grid, x2_grid, sigma1, sigma2, rho, dx1, dx2);

       c1_vec = [c1_vec; coeffs1']; c1_col = [c1_col; coeffs_col1']; rows_1 = [rows_1; A_row*ones(size(coeffs1'))];
       c2_vec = [c2_vec; coeffs2']; c2_col = [c2_col; coeffs_col2']; rows_2 = [rows_2; A_row*ones(size(coeffs2'))];
       c12_vec = [c12_vec; coeffs12']; c12_col = [c12_col; coeffs_col12']; rows_12 = [rows_12; A_row*ones(size(coeffs12'))];
       
       % reseni operatoru L1
       %A(A_row, coeffs_col1) = A(A_row, coeffs_col1) + coeffs1;
       % reseni operatoru L2
       %A(A_row, coeffs_col2) = A(A_row, coeffs_col2) + coeffs2;
       % reseni operatoru L12
       %A(A_row, coeffs_col12) = A(A_row, coeffs_col12) + coeffs12;
       
   end
end

A = sparse(rows_1, c1_col, c1_vec)+sparse(rows_2, c2_col, c2_vec)+sparse(rows_12, c12_col, c12_vec);
end

function [coeffs, coeffs_col] = lcu1(j1, j2, x1_grid, x2_grid, kappa1, theta1, sigma1, a1, lambda, J_mu, dx1)
% koeficienty vzhledem k procesu x1
% proces x1 ma tvar: dx1(t) = mu1(t,x1,x2)*dt + sigma1(t,x1,x2)*dW(1)
get_idx = @(j1, j2)( (j2 - 1)*numel(x1_grid)+j1); % funkce pro mapovani indexu do sloupcu matice A
dx = dx1; 
x1 = x1_grid(j1); 
x2 = x2_grid(j2);

mu = kappa1*(theta1 - x1)-a1*lambda*J_mu; % hodnota mu = mu1(t, x1, x2)
sigma_sq = sigma1^2; % ctverec difuzni slozku u dW(1), jedna se tedy o (sigma1(t,x1,x2))^2
diskont = exp(x1)+exp(x2); % sazba, ktera stoji v PDE vedle vyrazu V(t,x1,x2), jakoby diskontni sazba (bude stejne ve fcich lcu1 i lcu2)

l = -mu/(2*dx) + sigma_sq/(2*dx^2);
c = -sigma_sq/(dx^2) - 0.5*diskont;
u = mu/(2*dx) + sigma_sq/(2*dx^2);

coeffs = [l c u];
coeffs_col = [get_idx(j1-1, j2), get_idx(j1, j2) get_idx(j1+1, j2)];

%% pokud jsme na dolnim boundary x1 = x1_1, tak nastavime fwd diference s chybou O(h^2). Zde jsou koeficienty
if (j1 == 1)
    coeffs(1) = -3/2*mu/dx + sigma_sq/dx^2 -0.5*diskont;   coeffs_col(1) = get_idx(j1, j2);
    coeffs(2) = 2*mu/dx - 5/2*sigma_sq/dx^2;               coeffs_col(2) = get_idx(j1+1, j2);
    coeffs(3) = -mu/(2*dx) + 2*sigma_sq/dx^2;              coeffs_col(3) = get_idx(j1+2, j2);
    coeffs(4) = -1/2*sigma_sq/dx^2;                        coeffs_col(4) = get_idx(j1+3, j2);
end

%% pokud jsme na hornim boundary x1 = x1_m1, tak nastavime zpetne diference s chybou O(h^2). Zde jsou koeficienty
if (j1 == numel(x1_grid)) 
    coeffs(1) = -1/2*sigma_sq/dx^2;                        coeffs_col(1) = get_idx(j1-3, j2);
    coeffs(2) = mu/(2*dx) + 2*sigma_sq/dx^2;               coeffs_col(2) = get_idx(j1-2, j2);
    coeffs(3) = -2*mu/dx - 5/2*sigma_sq/dx^2;              coeffs_col(3) = get_idx(j1-1, j2);
    coeffs(4) = 3/2*mu/dx + sigma_sq/dx^2 -0.5*diskont;    coeffs_col(4) = get_idx(j1, j2);
end

end

function [coeffs, coeffs_col] = lcu2(j1, j2, x1_grid, x2_grid, kappa2, theta2, sigma2, a2, lambda, J_mu, dx2)
% koeficienty vzhledem k procesu x2
% proces x2 ma tvar: dx2(t) = mu2(t,x1,x2)*dt + sigma2(t,x1,x2)*dW(2)
get_idx = @(j1, j2)( (j2 - 1)*numel(x1_grid)+j1); % funkce pro mapovani indexu do sloupcu matice A
dx = dx2; 
x1 = x1_grid(j1); 
x2 = x2_grid(j2);

mu = kappa2*(theta2 - x2)-a2*lambda*J_mu; % hodnota mu = mu2(t, x1, x2)
sigma_sq = sigma2^2; % ctverec difuzni slozku u dW(2), jedna se tedy o (sigma2(t,x1,x2))^2
diskont = exp(x1)+exp(x2); % sazba, ktera stoji v PDE vedle vyrazu V(t,x1,x2), jakoby diskontni sazba (bude stejne ve fcich lcu1 i lcu2)

l = -mu/(2*dx) + sigma_sq/(2*dx^2);
c = -sigma_sq/(dx^2) - 0.5*diskont;
u = mu/(2*dx) + sigma_sq/(2*dx^2);

coeffs = [l c u];
coeffs_col = [get_idx(j1, j2-1) get_idx(j1, j2) get_idx(j1, j2+1)]; 


%% pokud jsme na dolnim boundary x2 = x2_1, tak nastavime fwd diference s chybou O(h^2). Zde jsou koeficienty
if (j2 == 1)
    coeffs(1) = -3/2*mu/dx + sigma_sq/dx^2 -0.5*diskont;   coeffs_col(1) = get_idx(j1, j2);
    coeffs(2) = 2*mu/dx - 5/2*sigma_sq/dx^2;               coeffs_col(2) = get_idx(j1, j2+1);
    coeffs(3) = -mu/(2*dx) + 2*sigma_sq/dx^2;              coeffs_col(3) = get_idx(j1, j2+2);
    coeffs(4) = -1/2*sigma_sq/dx^2;                        coeffs_col(4) = get_idx(j1, j2+3);
end

%% pokud jsme na hornim boundary x2 = x2_m2, tak nastavime zpetne diference s chybou O(h^2). Zde jsou koeficienty
if (j2 == numel(x2_grid))
    coeffs(1) = -1/2*sigma_sq/dx^2;                        coeffs_col(1) = get_idx(j1, j2-3);
    coeffs(2) = mu/(2*dx) + 2*sigma_sq/dx^2;               coeffs_col(2) = get_idx(j1, j2-2);
    coeffs(3) = -2*mu/dx - 5/2*sigma_sq/dx^2;              coeffs_col(3) = get_idx(j1, j2-1);
    coeffs(4) = 3/2*mu/dx + sigma_sq/dx^2 -0.5*diskont;    coeffs_col(4) = get_idx(j1, j2);
end

end

function [coeffs, coeffs_col] = lcu12(j1, j2, x1_grid, x2_grid, sigma1, sigma2, rho, dx1, dx2)
% koeficienty u smisene derivace
% zde pripadne upravit jen tento vyraz z(t,x1, x2) = rho(t, x1, x2)*sigma1(t,x1, x2)*sigma2(t,x1, x2), ktery je u smisene derivace
z = rho * sigma1*sigma2;

get_idx = @(j1, j2)( (j2 - 1)*numel(x1_grid)+j1); % funkce pro mapovani indexu do sloupcu matice A
%% operator L12 pro rohove body mrizky
if (j1 == 1 && j2 == 1) %levy horni roh (male x1, male x2), fwd x1, fwd x2
    coeffs = [9 -12 3 -12 16 -4 3 -4 1];
    coeffs_col = get_idx(j1+[0 0 0 1 1 1 2 2 2], j2+[0 1 2 0 1 2 0 1 2]);
end

if (j1 == numel(x1_grid) && j2 == 1) % levy dolni roh (velke x1, male x2), bkw x1, fwd x2
    coeffs = [-3 4 -1 12 -16 4 -9 12 -3];
    coeffs_col = get_idx(j1+[-2 -2 -2 -1 -1 -1 0 0 0], j2+[0 1 2 0 1 2 0 1 2]);
end

if (j1 == numel(x1_grid) && j2 == numel(x2_grid)) % pravy dolni roh (velke x1, velke x2), bkw x1, bkw x2
    coeffs = [1 -4 3 -4 16 -12 3 -12 9];
    coeffs_col = get_idx(j1+[-2 -2 -2 -1 -1 -1 0 0 0], j2+[-2 -1 0 -2 -1 0 -2 -1 0]);
end

if (j1 == 1 && j2 == numel(x2_grid)) % pravy horni roh (male x1, velke x2), fwd x1, bkw x2
    coeffs = [-3, 4, -1, 12, -16, 4, -9, 12, -3];
    coeffs_col = get_idx(j1+[0, 1, 2, 0, 1, 2, 0, 1, 2], j2+[-2, -2, -2, -1, -1, -1, 0, 0, 0]);
end

%% operator L12 pro hrany mrizky (bez rohu)
if (j2 == 1 && j1 ~= 1 && j1 ~= numel(x1_grid)) % leva hrana (male x2 a j1 ruzne od 1 a m1), c x1, fwd x2
    coeffs = [-3,4,-1,3,-4,1];
    coeffs_col = get_idx(j1+[1,1,1,-1,-1,-1], j2+[0,1,2,0,1,2]);
end

if (j2 == numel(x2_grid) && j1 ~= 1 && j1 ~= numel(x1_grid)) % prava hrana (velke x2 a j1 ruzne od 1 a m1), c x1, bkw x2
    coeffs = [1,-4,3,-1,4,-3];
    coeffs_col = get_idx(j1+[1,1,1,-1,-1,-1], j2+[-2,-1,0,-2,-1,0]);
end

if (j1 == 1 && j2 ~= 1 && j2 ~= numel(x2_grid)) % horni hrana (male x1 a j2 ruzne od 1 a m2), fwd x1, c x2, 
    coeffs = [-3,4,-1,3,-4,1];
    coeffs_col = get_idx(j1+[0,1,2,0,1,2], j2+[1,1,1,-1,-1,-1]);
end

if (j1 == numel(x1_grid) && j2 ~= 1 && j2 ~= numel(x2_grid)) % dolni hrana (velke x1 a j2 ruzne od 1 a m2), bkw x1, c x2, 
    coeffs = [1,-4,3,-1,4,-3];
    coeffs_col = get_idx(j1+[-2,-1,0,-2,-1,0], j2+[1,1,1,-1,-1,-1]);
end

%% operator L12 pro vnitrni body gridu (ani hrany mrizky ani rohy)
if (j1 ~= 1 && j1 ~= numel(x1_grid) && j2 ~= 1 && j2 ~= numel(x2_grid)) % vsechny ostatni pripady, tj. vnitrek gridu. c x1, c x2
    coeffs = [1 -1 -1 1];
    coeffs_col = get_idx(j1+[1 1 -1 -1], j2+[1 -1 1 -1]);
end
coeffs = coeffs * z/(4*dx1*dx2);

end