function [CP_risk, rf_value] = counterparty_risk_CDS_CIR_CIR_tmp(K, r, T, delta, delta_CP, gamma2, gamma_delta, kappa2, theta2, sigma2, kappa_delta, theta_delta, sigma_delta, rho, m1, m2, n, theta_scheme)
% funkce pocita riziko protistrany (CVA) u CDS obchodu. Intenzita referencniho jmena i protistrany se ridi korelovanymi (log)CIR procesy

% pozor (!!!), v kodu x1 znamena x2; a x2 znamena delta. Toto pretypovani vzniklo v dusledku fixovani nazvu promennych
% priklad: [CP_risk, rf_value] = counterparty_risk_CDS_CIR_CIR_tmp(0.02, 0.05, 5, 0.4, 0.4, 0.015, 0.017, 0.2, 0.04, 0.08, 0.25, 0.038, 0.07, 0.5, 100, 100, 50, 0.5)

x1 = log(gamma2);
x2 = log(gamma_delta);
x1_floor    = -7;   % min hodnota log intenzity referencniho jmeno u podkladoveho CDS
x1_ceil     =  0.5; % max hodnota log intenzity referencniho jmeno u podkladoveho CDS
x2_floor    = -7;   % min hodnota log intenzity intenzity protistrany
x2_ceil     =  0.5; % max hodnota log intenzity intenzity protistrany

dx1 = (x1_ceil-x1_floor)/(m1+1);
x1_grid = x1_floor+dx1:dx1:x1_ceil-dx1;
dx2 = (x2_ceil-x2_floor)/(m2+1);
x2_grid = x2_floor+dx2:dx2:x2_ceil-dx2;

dt = T/n;
t = 0:dt:T;

V = zeros(m1*m2,n+1);
x1_grid_rep = repmat(x1_grid(:), 1, m2); x1_grid_rep = x1_grid_rep(:);
x2_grid_rep = repmat(x2_grid(:)',m1, 1); x2_grid_rep = x2_grid_rep(:);
A = get_A(x1_grid, x2_grid, kappa2, theta2, sigma2, kappa_delta, theta_delta, sigma_delta, rho, r, dx1, dx2);

I = speye(m1*m2);
RHS_tmp = I+dt*(1-theta_scheme)*A;
LHS = I-dt*(1-theta_scheme)*A;
for i = numel(t)-1:-1:1
   rf_value = CDS_PDE_CIR_log(K, r, T-(t(i)+t(i+1))/2, delta, exp(x1_grid_rep), kappa2, theta2, sigma2, 100, 50);
   RHS = RHS_tmp*V(:, i+1) + dt*(exp(x2_grid_rep).*(1-delta_CP).*rf_value.*(rf_value>0));
   [V(:, i), ~] = bicgstab(LHS, RHS);
end

surf(x2_grid, x1_grid, reshape(V(:, 1)*10000, [m1, m2])); xlabel('CP log-intensity (\gamma_\delta)'); ylabel('ref. name log-intensity log(\gamma)'); zlabel('counterpaty credit risk (bps)');
colormap hsv; axis tight;
% V = reshape(V(:, 1)*10000, [m1, m2]);
% surf(x2_grid(20:end), x1_grid(1:end-40), V(1:end-40,20:end)); view(-150,32); axis tight;
% figure(2);
% plot(V(20, :));
% figure(3);
% surf(x2_grid(20:end), x1_grid(13:25), V(13:25,20:end));
CP_risk = interp2(x2_grid, x1_grid, reshape(V(:, 1), [m1, m2]), x2, x1);
rf_value = CDS_PDE_CIR_log(K, r, T, delta, gamma2, kappa2, theta2, sigma2, 80, 50);
end

function A = get_A(x1_grid, x2_grid, kappa2, theta2, sigma2, kappa_delta, theta_delta, sigma_delta, rho, r, dx1, dx2)

x1_grid = x1_grid(:);
x2_grid = x2_grid(:);

m1 = numel(x1_grid);
m2 = numel(x2_grid);

j1_rep = repmat( (1:m1)', 1, m2); j1_rep = j1_rep(:);
j2_rep = repmat( (1:m2), m1, 1); j2_rep = j2_rep(:);

[coeffs1, cols1, rows1] = lcu1(j1_rep, j2_rep, x1_grid, x2_grid,  kappa2, theta2, sigma2, r, dx1);
[coeffs2, cols2, rows2] = lcu2(j1_rep, j2_rep, x1_grid, x2_grid, kappa_delta, theta_delta, sigma_delta, r, dx2);
[coeffs12, cols12, rows12] = lcu12(j1_rep, j2_rep, x1_grid, x2_grid, sigma2, sigma_delta, rho, dx1, dx2);

A = sparse(rows1, cols1, coeffs1) + sparse(rows2, cols2, coeffs2) + sparse(rows12, cols12, coeffs12);

end

function [coeffs, cols, rows] = lcu1(j1, j2, x1_grid, x2_grid, kappa2, theta2, sigma2, r, dx1)
% koeficienty vzhledem k procesu x1
% proces x1 ma tvar: dx1(t) = mu1(t,x1,x2)*dt + sigma1(t,x1,x2)*dW(1)
get_idx = @(j1, j2)( (j2 - 1)*numel(x1_grid)+j1); % funkce pro mapovani indexu do sloupcu matice A
dx = dx1; 
x1 = x1_grid(j1); 
x2 = x2_grid(j2);

mu = exp(-x1).*(kappa2*(theta2 - exp(x1)) - 1/2 *sigma2^2); % hodnota mu = mu1(t, x1, x2)
sigma_sq = sigma2^2*exp(-x1); % ctverec difuzni slozku u dW(1), jedna se tedy o (sigma1(t,x1,x2))^2
diskont = r+exp(x2); % sazba, ktera stoji v PDE vedle vyrazu V(t,x1,x2), jakoby diskontni sazba (bude stejne ve fcich lcu1 i lcu2)

non_boundary_idx = (j1> 1 & j1 < numel(x1_grid));
l = -mu(non_boundary_idx)/(2*dx) + sigma_sq(non_boundary_idx)/(2*dx^2);
c = -sigma_sq(non_boundary_idx)/(dx^2) - 0.5*diskont(non_boundary_idx);
u = mu(non_boundary_idx)/(2*dx) + sigma_sq(non_boundary_idx)/(2*dx^2);

coeffs = [l(:) c(:) u(:)];
coeffs_col = [get_idx(j1(non_boundary_idx)-1, j2(non_boundary_idx)), get_idx(j1(non_boundary_idx), j2(non_boundary_idx)) get_idx(j1(non_boundary_idx)+1, j2(non_boundary_idx))];
coeffs_rows = repmat(get_idx(j1(non_boundary_idx), j2(non_boundary_idx)), 1, 3);

%% pokud jsme na dolnim boundary x1 = x1_1, tak nastavime fwd diference s chybou O(h^2). Zde jsou koeficienty
idx_low = j1==1;

coeffs_low(:,1) = -3/2*mu(idx_low)/dx + sigma_sq(idx_low)/dx^2 -0.5*diskont(idx_low);
coeffs_low(:,2) = 2*mu(idx_low)/dx - 5/2*sigma_sq(idx_low)/dx^2;
coeffs_low(:,3) = -mu(idx_low)/(2*dx) + 2*sigma_sq(idx_low)/dx^2; 
coeffs_low(:,4) = -1/2*sigma_sq(idx_low)/dx^2;
coeffs_low_col = [ get_idx(j1(idx_low), j2(idx_low)), get_idx(j1(idx_low)+1, j2(idx_low)), get_idx(j1(idx_low)+2, j2(idx_low)), get_idx(j1(idx_low)+3, j2(idx_low))];
rows_low =  repmat(get_idx(j1(idx_low), j2(idx_low)), 1, 4);

%% pokud jsme na hornim boundary x1 = x1_m1, tak nastavime zpetne diference s chybou O(h^2). Zde jsou koeficienty

idx_high = (j1 == numel(x1_grid));
coeffs_high(:,1) = -1/2*sigma_sq(idx_high)/dx^2;
coeffs_high(:,2) = mu(idx_high)/(2*dx) + 2*sigma_sq(idx_high)/dx^2; 
coeffs_high(:,3) = -2*mu(idx_high)/dx - 5/2*sigma_sq(idx_high)/dx^2;
coeffs_high(:,4) = 3/2*mu(idx_high)/dx + sigma_sq(idx_high)/dx^2 -0.5*diskont(idx_high);
coeffs_high_col = [ get_idx(j1(idx_high)-3, j2(idx_high)), get_idx(j1(idx_high)-2, j2(idx_high)), get_idx(j1(idx_high)-1, j2(idx_high)), get_idx(j1(idx_high), j2(idx_high))];
rows_high =  repmat(get_idx(j1(idx_high), j2(idx_high)), 1, 4);

boundary_coeffs = [coeffs_low; coeffs_high];
boundary_col    = [coeffs_low_col; coeffs_high_col];
rows_boundary   = [rows_low; rows_high];

coeffs = [coeffs(:); boundary_coeffs(:)];
cols = [coeffs_col(:); boundary_col(:)];
rows = [coeffs_rows(:); rows_boundary(:)];

end

function [coeffs, cols, rows] = lcu2(j1, j2, x1_grid, x2_grid, kappa_delta, theta_delta, sigma_delta, r, dx2)
% koeficienty vzhledem k procesu x2
% proces x2 ma tvar: dx2(t) = mu2(t,x1,x2)*dt + sigma2(t,x1,x2)*dW(2)
get_idx = @(j1, j2)( (j2 - 1)*numel(x1_grid)+j1); % funkce pro mapovani indexu do sloupcu matice A
dx = dx2; 
x1 = x1_grid(j1); 
x2 = x2_grid(j2);

mu = exp(-x2).*(kappa_delta*(theta_delta - exp(x2)) - 1/2 *sigma_delta^2); % hodnota mu = mu2(t, x1, x2)
sigma_sq = sigma_delta^2*exp(-x2); % ctverec difuzni slozku u dW(2), jedna se tedy o (sigma2(t,x1,x2))^2
diskont = r+exp(x2); % sazba, ktera stoji v PDE vedle vyrazu V(t,x1,x2), jakoby diskontni sazba (bude stejne ve fcich lcu1 i lcu2)

non_boundary_idx = (j2> 1 & j2 < numel(x2_grid));
l = -mu(non_boundary_idx)/(2*dx) + sigma_sq(non_boundary_idx)/(2*dx^2);
c = -sigma_sq(non_boundary_idx)/(dx^2) - 0.5*diskont(non_boundary_idx);
u = mu(non_boundary_idx)/(2*dx) + sigma_sq(non_boundary_idx)/(2*dx^2);

coeffs = [l(:) c(:) u(:)];
coeffs_col = [get_idx(j1(non_boundary_idx), j2(non_boundary_idx)-1) get_idx(j1(non_boundary_idx), j2(non_boundary_idx)) get_idx(j1(non_boundary_idx), j2(non_boundary_idx)+1)]; 
coeffs_rows = repmat(get_idx(j1(non_boundary_idx), j2(non_boundary_idx)), 1, 3);

%% pokud jsme na dolnim boundary x2 = x2_1, tak nastavime fwd diference s chybou O(h^2). Zde jsou koeficienty
idx_low = j2==1;

coeffs_low(:,1) = -3/2*mu(idx_low)/dx + sigma_sq(idx_low)/dx^2 -0.5*diskont(idx_low);
coeffs_low(:,2) = 2*mu(idx_low)/dx - 5/2*sigma_sq(idx_low)/dx^2;
coeffs_low(:,3) = -mu(idx_low)/(2*dx) + 2*sigma_sq(idx_low)/dx^2; 
coeffs_low(:,4) = -1/2*sigma_sq(idx_low)/dx^2;
coeffs_low_col = [ get_idx(j1(idx_low), j2(idx_low)), get_idx(j1(idx_low), j2(idx_low)+1), get_idx(j1(idx_low), j2(idx_low)+2), get_idx(j1(idx_low), j2(idx_low)+3)];
rows_low =  repmat(get_idx(j1(idx_low), j2(idx_low)), 1, 4);

%% pokud jsme na hornim boundary x2 = x2_m2, tak nastavime zpetne diference s chybou O(h^2). Zde jsou koeficienty

idx_high = (j2 == numel(x2_grid));
coeffs_high(:,1) = -1/2*sigma_sq(idx_high)/dx^2;
coeffs_high(:,2) = mu(idx_high)/(2*dx) + 2*sigma_sq(idx_high)/dx^2; 
coeffs_high(:,3) = -2*mu(idx_high)/dx - 5/2*sigma_sq(idx_high)/dx^2;
coeffs_high(:,4) = 3/2*mu(idx_high)/dx + sigma_sq(idx_high)/dx^2 -0.5*diskont(idx_high);
coeffs_high_col = [ get_idx(j1(idx_high), j2(idx_high)-3), get_idx(j1(idx_high), j2(idx_high)-2), get_idx(j1(idx_high), j2(idx_high)-1), get_idx(j1(idx_high), j2(idx_high))];
rows_high =  repmat(get_idx(j1(idx_high), j2(idx_high)), 1, 4);

boundary_coeffs = [coeffs_low; coeffs_high];
boundary_col    = [coeffs_low_col; coeffs_high_col];
rows_boundary   = [rows_low; rows_high];

coeffs = [coeffs(:); boundary_coeffs(:)];
cols = [coeffs_col(:); boundary_col(:)];
rows = [coeffs_rows(:); rows_boundary(:)];
end

function [coeffs, cols, rows] = lcu12(j1, j2, x1_grid, x2_grid, sigma2, sigma_delta, rho, dx1, dx2)
% koeficienty u smisene derivace
% zde pripadne upravit jen tento vyraz z(t,x1, x2) = rho(t, x1, x2)*sigma1(t,x1, x2)*sigma2(t,x1, x2), ktery je u smisene derivace
x1 = x1_grid(j1); 
x2 = x2_grid(j2);

z = rho * sigma2 *sigma_delta * sqrt(exp(-x1)) .* sqrt(exp(-x2));

get_idx = @(j1, j2)( (j2 - 1)*numel(x1_grid)+j1); % funkce pro mapovani indexu do sloupcu matice A
%% operator L12 pro rohove body mrizky

%levy horni roh (male x1, male x2), fwd x1, fwd x2
idx = ((j1 == 1) & (j2 == 1));
col9 = get_idx(j1(idx)+[0 0 0 1 1 1 2 2 2], j2(idx)+[0 1 2 0 1 2 0 1 2]);
row9 = repmat(get_idx(j1(idx), j2(idx)), 1, 9);
coeff9 = [9 -12 3 -12 16 -4 3 -4 1].*z(idx)/(4*dx1*dx2); 

% levy dolni roh (velke x1, male x2), bkw x1, fwd x2
idx = ((j1 == numel(x1_grid)) & (j2 == 1));
col9 = [col9; get_idx(j1(idx)+[-2 -2 -2 -1 -1 -1 0 0 0], j2(idx)+[0 1 2 0 1 2 0 1 2])];
row9 = [row9; repmat(get_idx(j1(idx), j2(idx)), 1, 9)];
coeff9 = [coeff9; [-3 4 -1 12 -16 4 -9 12 -3].*z(idx)/(4*dx1*dx2)];

% pravy dolni roh (velke x1, velke x2), bkw x1, bkw x2
idx = ((j1 == numel(x1_grid)) & (j2 == numel(x2_grid)));
col9 = [col9; get_idx(j1(idx)+[-2 -2 -2 -1 -1 -1 0 0 0], j2(idx)+[-2 -1 0 -2 -1 0 -2 -1 0])];
row9 = [row9; repmat(get_idx(j1(idx), j2(idx)), 1, 9)];
coeff9 = [coeff9; [1 -4 3 -4 16 -12 3 -12 9].*z(idx)/(4*dx1*dx2)];


% pravy horni roh (male x1, velke x2), fwd x1, bkw x2
idx = ((j1 == 1) & (j2 == numel(x2_grid)));
col9 = [col9; get_idx(j1(idx)+[0, 1, 2, 0, 1, 2, 0, 1, 2], j2(idx)+[-2, -2, -2, -1, -1, -1, 0, 0, 0])];
row9 = [row9; repmat(get_idx(j1(idx), j2(idx)), 1, 9)];
coeff9 = [coeff9; [-3, 4, -1, 12, -16, 4, -9, 12, -3]*z(idx)/(4*dx1*dx2)];


%% operator L12 pro hrany mrizky (bez rohu)
% leva hrana (male x2 a j1 ruzne od 1 a m1), c x1, fwd x2
idx = ((j2 == 1) & (j1 ~= 1) & (j1 ~= numel(x1_grid)));
col6 = get_idx(repmat(j1(idx), 1, 6)+repmat([1,1,1,-1,-1,-1], sum(idx),1), repmat(j2(idx), 1, 6)+repmat([0,1,2,0,1,2], sum(idx), 1));
row6 = repmat(get_idx(j1(idx), j2(idx)), 1, 6);
coeff6 = repmat([-3,4,-1,3,-4,1], sum(idx), 1).*repmat(z(idx), 1, 6)/(4*dx1*dx2); 


% prava hrana (velke x2 a j1 ruzne od 1 a m1), c x1, bkw x2
idx = ((j2 == numel(x2_grid)) & (j1 ~= 1) & (j1 ~= numel(x1_grid)));
col6 = [col6; get_idx(repmat(j1(idx), 1, 6)+repmat([1,1,1,-1,-1,-1], sum(idx),1), repmat(j2(idx), 1, 6)+repmat([-2,-1,0,-2,-1,0], sum(idx), 1))];
row6 = [row6; repmat(get_idx(j1(idx), j2(idx)), 1, 6)];
coeff6 = [coeff6; repmat([1,-4,3,-1,4,-3], sum(idx), 1).*repmat(z(idx), 1, 6)/(4*dx1*dx2)]; 

% horni hrana (male x1 a j2 ruzne od 1 a m2), fwd x1, c x2
idx = ((j1 == 1) & (j2 ~= 1) & (j2 ~= numel(x2_grid)));
col6 = [col6; get_idx(repmat(j1(idx), 1, 6)+repmat([0,1,2,0,1,2], sum(idx),1), repmat(j2(idx), 1, 6)+repmat([1,1,1,-1,-1,-1], sum(idx), 1))];
row6 = [row6; repmat(get_idx(j1(idx), j2(idx)), 1, 6)];
coeff6 = [coeff6; repmat([-3,4,-1,3,-4,1], sum(idx), 1).*repmat(z(idx), 1, 6)/(4*dx1*dx2)]; 

% dolni hrana (velke x1 a j2 ruzne od 1 a m2), bkw x1, c x2
idx = ((j1 == numel(x1_grid)) & (j2 ~= 1) & (j2 ~= numel(x2_grid)));
col6 = [col6; get_idx(repmat(j1(idx), 1, 6)+repmat([-2,-1,0,-2,-1,0], sum(idx),1), repmat(j2(idx), 1, 6)+repmat([1,1,1,-1,-1,-1], sum(idx), 1))];
row6 = [row6; repmat(get_idx(j1(idx), j2(idx)), 1, 6)];
coeff6 = [coeff6; repmat([1,-4,3,-1,4,-3], sum(idx), 1).*repmat(z(idx), 1, 6)/(4*dx1*dx2)];


%% operator L12 pro vnitrni body gridu (ani hrany mrizky ani rohy)
idx = ((j1 ~= 1) & (j1 ~= numel(x1_grid)) & (j2 ~= 1) & (j2 ~= numel(x2_grid)));
col4 = get_idx(repmat(j1(idx), 1, 4)+repmat([1 1 -1 -1], sum(idx),1), repmat(j2(idx), 1, 4)+repmat([1 -1 1 -1], sum(idx), 1));
row4 = repmat(get_idx(j1(idx), j2(idx)), 1, 4);
coeff4 = repmat([1 -1 -1 1], sum(idx), 1).*repmat(z(idx), 1, 4)/(4*dx1*dx2); 

coeffs = [coeff4(:); coeff6(:); coeff9(:)];
cols   = [col4(:)  ; col6(:);   col9(:)  ];
rows   = [row4(:)  ; row6(:);   row9(:)  ];


end